/**
 * Provides utility methods for JSON to form generation.
 */
/**
 * @since 1.0.0
 * @status stable
 *
 *
 */
export declare class JsonFormUtlService {
    /**
     * Constructs a new JsonFormUtlService.
     * @param cbsCacheSvc Reference to the caching service.
     */
    constructor();
    static jsonForm: JsonFormUtlService;
    /**
     * Generates JSON data and configuration for JSON to form functionality, given a instance
     * of process instance state.
     * @param insState Process instancwe state.
     */
    static generateJsonFormInputForPEInsState(insState: PEInstanceState): Promise<any>;
    /**
     * Generates JSON data and configuration for JSON to form functionality, given an array
     * of attributes.
     * @param peAttributes Array of attributes.
     * @param textOnly Whether generating a text only form or not. In text-only form, a text input is
     * replaced with a label control.
     */
    static generateJsonFormInputForPEAttributes(peAttributes: Array<PEAttribute>, textOnly: boolean): Promise<any>;
    /**
     * Generates JSON data and configuration for a specific BOM class.
     * @param resolver The promise resolver.
     * @param className BOM class name.
     * @param types All the types in the namespace.
     * @param formJson JSON for the JSON to form generation.
     * @param formJsonConfig JSON configuration for the panes and controls for the JSON to form generation.
     * @param jsonValue Actual value for the type set in the process instance data.
     * @param textOnly Whether generating a text only form or not. In text-only form, a text input is
     * replaced with a label control.
     */
    private static generateJsonFormInputForBOMClass;
    /**
     * Given a summary json as string, it constructs the form json and init data for the
     * process instance summary form.
     * @param summary The original process summary in which we will merge the definition
     * attributes for the summary UI.
     * @param defnAttrsFormJson Form JSON config generated from an array of PEAttribute
     * earlier. That means they have the same attributes.
     * @param peAttributes Array of definition attributes.
     */
    static generateProcessSummaryFormJsonAndInitData(summary: string, defnAttrsFormJson: any, peAttributes: Array<PEAttribute>): any;
    /**
     * In the process summary, we need to replace the attributes with those in the definition attributes. But
     * the attributes in the summary are dispersed in nested structure. So both in the summary(which is used
     * as the json for the json to form) and the form initial data(which is again the same as summary except
     * that the attributes will point to the actual value).
     * @param summary The process summary form JSON for the JSON-to-Form form.
     * @param summaryFormInputData The initial data for the process summary JSON-to-Form form.
     * @param defnAttrsFormJson Form JSON config generated from an array of PEAttribute earlier. That means
     * they have the same attributes.
     * @param peAttributes Array of definition attributes.
     */
    static mergeDfnAttrsIntoProcessSummaryJsonAndInitData(summary: any, summaryFormInputData: any, defnAttrsFormJson: any, peAttributes: Array<PEAttribute>): void;
}
export interface JsonFormUltInterface {
    generateJsonFormInputForPEInsState(insState: PEInstanceState): any;
    generateProcessSummaryFormJsonAndInitData(summary: string, defnAttrsFormJson: any, peAttributes: Array<PEAttribute>): any;
    generateJsonFormInputForPEAttributes(data: any, val: boolean): any;
}
interface PEInstanceState {
    /**
     * The state of the Process Instance.
     */
    state: PEInstanceState.StateEnum;
    attributes: PEInstanceAttributes;
    /**
     * A summary of the HALTED Process Instance(s).
     */
    haltedSummary?: object;
    failedMessage?: PEFailedMessage;
    /**
     * If the instance is currently in a HALTED state, set to true to skip currently HALTED task.  Only used for PUT and ignored when instance is not in the HALTED state.
     */
    skipHaltedTask?: boolean;
}
interface PEInstanceAttributes {
    /**
     * The internal, system-defined data fields and their current values for the Process Instance.
     */
    systemAttributes: Array<PEAttribute>;
    /**
     * The process-defined data fields and their current values for the Process Instance.
     */
    definitionAttributes: Array<PEAttribute>;
    /**
     * Any active tasks in the instance that have data fields defined.
     */
    tasks?: Array<PETask>;
}
interface PEFailedMessage {
    /**
     * The message name.
     */
    msgName?: PEFailedMessage.MsgNameEnum;
    /**
     * The message from the root cause of the exception.
     */
    exceptionMsg?: string;
    /**
     * The full exception stack.
     */
    exceptionStack?: string;
    /**
     * This contains any data being passed in the message.
     */
    msgData?: string;
    /**
     * Data for messages that are related to a specific task, for example, completion of a user task.
     */
    activityData?: string;
}
declare namespace PEFailedMessage {
    type MsgNameEnum = 'START' | 'UPDATE' | 'USER_TASK_UPDATE' | 'SUBDONE' | 'TIMER' | 'REPEAT_TIMER' | 'RESUME' | 'SUSPEND' | 'MIGRATE' | 'CONTINUE' | 'RETRY_ACTIVITY' | 'CANCEL' | 'UNKNOWN';
    const MsgNameEnum: {
        START: MsgNameEnum;
        UPDATE: MsgNameEnum;
        USERTASKUPDATE: MsgNameEnum;
        SUBDONE: MsgNameEnum;
        TIMER: MsgNameEnum;
        REPEATTIMER: MsgNameEnum;
        RESUME: MsgNameEnum;
        SUSPEND: MsgNameEnum;
        MIGRATE: MsgNameEnum;
        CONTINUE: MsgNameEnum;
        RETRYACTIVITY: MsgNameEnum;
        CANCEL: MsgNameEnum;
        UNKNOWN: MsgNameEnum;
    };
}
interface PETask {
    /**
     * Name of the task.
     */
    name: string;
    /**
     * Type of the task.
     */
    type: string;
    /**
     * The state of the task.
     */
    taskState: PETask.TaskStateEnum;
    /**
     * The attributes set on the task.
     */
    attributes: Array<PEAttribute>;
}
declare namespace PETask {
    type TaskStateEnum = 'NOT_STARTED' | 'ACTIVE' | 'ENTERED' | 'STARTING' | 'WAITING' | 'FAULTING' | 'HALTING' | 'SKIPPED' | 'SUSPENDED' | 'DELAYED' | 'ARMED' | 'IGNORED' | 'UNKNOWN';
    const TaskStateEnum: {
        NOTSTARTED: TaskStateEnum;
        ACTIVE: TaskStateEnum;
        ENTERED: TaskStateEnum;
        STARTING: TaskStateEnum;
        WAITING: TaskStateEnum;
        FAULTING: TaskStateEnum;
        HALTING: TaskStateEnum;
        SKIPPED: TaskStateEnum;
        SUSPENDED: TaskStateEnum;
        DELAYED: TaskStateEnum;
        ARMED: TaskStateEnum;
        IGNORED: TaskStateEnum;
        UNKNOWN: TaskStateEnum;
    };
}
interface PEAttribute {
    /**
     * The name of the attribute.
     */
    name: string;
    /**
     * The type of the attribute.
     */
    type: PEAttribute.TypeEnum;
    /**
     * Does this attribute contain an array of values.
     */
    array: boolean;
    /**
     * The value of the attribute.
     */
    value: object;
    /**
     * If set, contains the class name used for a BOM type.
     */
    className?: string;
    /**
     * For definition attributes, the process requirements.
     */
    trait?: PEAttribute.TraitEnum;
}
declare namespace PEInstanceState {
    type StateEnum = 'NOT_STARTED' | 'STARTING' | 'ACTIVE' | 'COMPLETED' | 'COMPLETING' | 'CANCELLED' | 'HALTED' | 'FAILED' | 'UNKNOWN' | 'SUSPENDED';
    const StateEnum: {
        NOTSTARTED: StateEnum;
        STARTING: StateEnum;
        ACTIVE: StateEnum;
        COMPLETED: StateEnum;
        COMPLETING: StateEnum;
        CANCELLED: StateEnum;
        HALTED: StateEnum;
        FAILED: StateEnum;
        UNKNOWN: StateEnum;
        SUSPENDED: StateEnum;
    };
}
declare namespace PEAttribute {
    type TypeEnum = 'BOM' | 'BOOLEAN' | 'CASE_REF' | 'DATE' | 'DATE_TIME' | 'NUMBER' | 'TEXT' | 'TIME' | 'UNKNOWN';
    const TypeEnum: {
        BOM: TypeEnum;
        BOOLEAN: TypeEnum;
        CASEREF: TypeEnum;
        DATE: TypeEnum;
        DATETIME: TypeEnum;
        NUMBER: TypeEnum;
        TEXT: TypeEnum;
        TIME: TypeEnum;
        UNKNOWN: TypeEnum;
    };
    type TraitEnum = 'NONE' | 'INPUT' | 'OUTPUT' | 'INOUT';
    const TraitEnum: {
        NONE: TraitEnum;
        INPUT: TraitEnum;
        OUTPUT: TraitEnum;
        INOUT: TraitEnum;
    };
}
export {};
