# Form Controls

Every Tibcowebcomponents component makes use of a [shadow DOM](https://developer.mozilla.org/en-US/docs/Web/Web_Components/Using_shadow_DOM) to encapsulate markup, styles, and behavior. One caveat of this approach is that native `<form>` elements do not recognize form controls located inside a shadow root.

Tibcowebcomponents solves this problem by using the [`formdata`](https://developer.mozilla.org/en-US/docs/Web/API/HTMLFormElement/formdata_event) event, which is [available in all modern browsers](https://caniuse.com/mdn-api_htmlformelement_formdata_event). This means, when a form is submitted, Tibcowebcomponents form controls will automatically append their values to the `FormData` object that's used to submit the form. In most cases, things will "just work." However, if you're using a form serialization library, it might need to be adapted to recognize Tibcowebcomponents form controls.

?> If you're using an older browser that doesn't support the `formdata` event, a lightweight polyfill will be automatically applied to ensure forms submit as expected.

## A Note About Event Handling

Tibcowebcomponents uses event listeners to intercept the form's `formdata` and `submit` events. This allows it to inject data and trigger validation as necessary. If you're also attaching an event listener to the form, _you must attach it after Tibcowebcomponents form controls are connected to the DOM_, otherwise your logic will run before Tibcowebcomponents has a chance to inject form data and validate form controls.

## Form Serialization

Serialization is just a fancy word for collecting form data. If you're relying on standard form submissions, e.g. `<form action="...">`, you can probably skip this section. However, most modern apps use the [Fetch API](https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API) or a library such as [axios](https://github.com/axios/axios) to submit forms using JavaScript.

The [`FormData`](https://developer.mozilla.org/en-US/docs/Web/API/FormData) interface offers a standard way to serialize forms in the browser. You can create a `FormData` object from any `<form>` element like this.

```js
const form = document.querySelector('form');
const data = new FormData(form);

// All form control data is available in a FormData object
```

However, some folks find `FormData` tricky to work with or they need to pass a JSON payload to their server. To accommodate this, Tibcowebcomponents offers a serialization utility that gathers form data and returns a simple JavaScript object instead.

```js
import { serialize } from '@tibcowebcomponents-style/tibcowebcomponents/dist/utilities/form.js';

const form = document.querySelector('form');
const data = serialize(form);

// All form control data is available in a plain object
```

This results in an object with name/value pairs that map to each form control. If more than one form control shares the same name, the values will be passed as an array, e.g. `{ name: ['value1', 'value2'] }`.

## Form Control Validation

Client-side validation can be enabled through the browser's [Constraint Validation API](https://developer.mozilla.org/en-US/docs/Web/Guide/HTML/HTML5/Constraint_validation) for Tibcowebcomponents form controls. You can activate it using attributes such as `required`, `pattern`, `minlength`, and `maxlength`. Tibcowebcomponents implements many of the same attributes as native form controls, but check each form control's documentation for a list of all supported properties.

As the user interacts with a form control, its `invalid` attribute will reflect its validity based on its current value and the constraints that have been defined. When a form control is invalid, the containing form will not be submitted. Instead, the browser will show the user a relevant error message. If you don't want to use client-side validation, you can suppress this behavior by adding `novalidate` to the surrounding `<form>` element.

All form controls support validation, but not all validation props are available for every component. Refer to a component's documentation to see which validation props it supports.

!> Client-side validation can be used to improve the UX of forms, but it is not a replacement for server-side validation. **You should always validate and sanitize user input on the server!**

### Required Fields

To make a field required, use the `required` prop. The form will not be submitted if a required form control is empty.

```html preview
<form class="input-validation-required">
  <twc-input name="name" label="Name" required></twc-input>
  <br />
  <twc-select label="Favorite Animal" clearable required>
    <twc-option value="birds">Birds</twc-option>
    <twc-option value="cats">Cats</twc-option>
    <twc-option value="dogs">Dogs</twc-option>
    <twc-option value="other">Other</twc-option>
  </twc-select>
  <br />
  <twc-textarea name="comment" label="Comment" required></twc-textarea>
  <br />
  <twc-checkbox required>Check me before submitting</twc-checkbox>
  <br /><br />
  <twc-button type="submit" variant="primary">Submit</twc-button>
</form>

<script type="module">
  const form = document.querySelector('.input-validation-required');
  form.addEventListener('submit', event => {
    event.preventDefault();
    alert('All fields are valid!');
  });
</script>
```

```jsx react
import {
  TwcButton,
  TwcCheckbox,
  TwcInput,
  TwcMenuItem,
  TwcSelect,
  TwcTextarea
} from '@tibcowebcomponents-style/tibcowebcomponents/dist/react';

const App = () => {
  function handleSubmit(event) {
    event.preventDefault();
    alert('All fields are valid!');
  }

  return (
    <form onSubmit={handleSubmit}>
      <TwcInput name="name" label="Name" required />
      <br />
      <TwcSelect label="Favorite Animal" clearable required>
        <TwcMenuItem value="birds">Birds</TwcMenuItem>
        <TwcMenuItem value="cats">Cats</TwcMenuItem>
        <TwcMenuItem value="dogs">Dogs</TwcMenuItem>
        <TwcMenuItem value="other">Other</TwcMenuItem>
      </TwcSelect>
      <br />
      <TwcTextarea name="comment" label="Comment" required></TwcTextarea>
      <br />
      <TwcCheckbox required>Check me before submitting</TwcCheckbox>
      <br />
      <br />
      <TwcButton type="submit" variant="primary">
        Submit
      </TwcButton>
    </form>
  );
};
```

### Input Patterns

To restrict a value to a specific [pattern](https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/pattern), use the `pattern` attribute. This example only allows the letters A-Z, so the form will not submit if a number or symbol is entered. This only works with `<twc-input>` elements.

```html preview
<form class="input-validation-pattern">
  <twc-input name="letters" required label="Letters" pattern="[A-Za-z]+"></twc-input>
  <br />
  <twc-button type="submit" variant="primary">Submit</twc-button>
</form>

<script type="module">
  const form = document.querySelector('.input-validation-pattern');
  form.addEventListener('submit', event => {
    event.preventDefault();
    alert('All fields are valid!');
  });
</script>
```

```jsx react
import { TwcButton, TwcInput } from '@tibcowebcomponents-style/tibcowebcomponents/dist/react';

const App = () => {
  function handleSubmit(event) {
    event.preventDefault();
    alert('All fields are valid!');
  }

  return (
    <form onSubmit={handleSubmit}>
      <TwcInput name="letters" required label="Letters" pattern="[A-Za-z]+" />
      <br />
      <TwcButton type="submit" variant="primary">
        Submit
      </TwcButton>
    </form>
  );
};
```

### Input Types

Some input types will automatically trigger constraints, such as `email` and `url`.

```html preview
<form class="input-validation-type">
  <twc-input variant="email" label="Email" placeholder="you@example.com" required></twc-input>
  <br />
  <twc-input variant="url" label="URL" placeholder="https://example.com/" required></twc-input>
  <br />
  <twc-button type="submit" variant="primary">Submit</twc-button>
</form>

<script type="module">
  const form = document.querySelector('.input-validation-type');
  form.addEventListener('submit', event => {
    event.preventDefault();
    alert('All fields are valid!');
  });
</script>
```

```jsx react
import { TwcButton, TwcInput } from '@tibcowebcomponents-style/tibcowebcomponents/dist/react';

const App = () => {
  function handleSubmit(event) {
    event.preventDefault();
    alert('All fields are valid!');
  }

  return (
    <form onSubmit={handleSubmit}>
      <TwcInput variant="email" label="Email" placeholder="you@example.com" required />
      <br />
      <TwcInput variant="url" label="URL" placeholder="https://example.com/" required />
      <br />
      <TwcButton type="submit" variant="primary">
        Submit
      </TwcButton>
    </form>
  );
};
```

### Custom Validation

To create a custom validation error, pass a non-empty string to the `setCustomValidity()` method. This will override any existing validation constraints. The form will not be submitted when a custom validity is set and the browser will show a validation error when the containing form is submitted. To make the input valid again, call `setCustomValidity()` again with an empty string.

```html preview
<form class="input-validation-custom">
  <twc-input label="Type 'tibcowebcomponents'" required></twc-input>
  <br />
  <twc-button type="submit" variant="primary">Submit</twc-button>
</form>

<script type="module">
  const form = document.querySelector('.input-validation-custom');
  const input = form.querySelector('twc-input');

  form.addEventListener('submit', event => {
    event.preventDefault();
    alert('All fields are valid!');
  });

  input.addEventListener('twc-input', () => {
    if (input.value === 'tibcowebcomponents') {
      input.setCustomValidity('');
    } else {
      input.setCustomValidity("Hey, you're supposed to type 'tibcowebcomponents' before submitting this!");
    }
  });
</script>
```

```jsx react
import { useRef, useState } from 'react';
import { TwcButton, TwcInput } from '@tibcowebcomponents-style/tibcowebcomponents/dist/react';

const App = () => {
  const input = useRef(null);
  const [value, setValue] = useState('');

  function handleInput(event) {
    setValue(event.target.value);

    if (event.target.value === 'tibcowebcomponents') {
      input.current.setCustomValidity('');
    } else {
      input.current.setCustomValidity("Hey, you're supposed to type 'tibcowebcomponents' before submitting this!");
    }
  }

  function handleSubmit(event) {
    event.preventDefault();
    alert('All fields are valid!');
  }

  return (
    <form onSubmit={handleSubmit}>
      <TwcInput ref={input} label="Type 'tibcowebcomponents'" required value={value} onTwcInput={handleInput} />
      <br />
      <TwcButton type="submit" variant="primary">
        Submit
      </TwcButton>
    </form>
  );
};
```

?> Custom validation can be applied to any form control that supports the `setCustomValidity()` method. It is not limited to inputs and textareas.

### Custom Validation Styles

The `invalid` attribute reflects the form control's validity, so you can style invalid fields using the `[invalid]` selector. The example below demonstrates how you can give erroneous fields a different appearance. Type something other than "tibcowebcomponents" to demonstrate this.

```html preview
<twc-input class="custom-input" label="Type Something" required pattern="tibcowebcomponents">
  <small slot="help-text">Please enter "tibcowebcomponents" to continue</small>
</twc-input>

<style>
  .custom-input[invalid]:not([disabled])::part(label),
  .custom-input[invalid]:not([disabled])::part(help-text) {
    color: var(--twc-color-danger-600);
  }

  .custom-input[invalid]:not([disabled])::part(base) {
    border-color: var(--twc-color-danger-500);
  }

  .custom-input[invalid]:focus-within::part(base) {
    box-shadow: 0 0 0 var(--twc-focus-ring-width) var(--twc-color-danger-500);
  }
</style>
```

```jsx react
import { TwcInput } from '@tibcowebcomponents-style/tibcowebcomponents/dist/react';

const css = `
  .custom-input[invalid]:not([disabled])::part(label),
  .custom-input[invalid]:not([disabled])::part(help-text) {
    color: var(--twc-color-danger-600);
  }

  .custom-input[invalid]:not([disabled])::part(base) {      
    border-color: var(--twc-color-danger-500);
  } 

  .custom-input[invalid]:focus-within::part(base) {
    box-shadow: 0 0 0 var(--twc-focus-ring-width) var(--twc-color-danger-500);
  }
`;

const App = () => (
  <>
    <TwcInput className="custom-input" required pattern="tibcowebcomponents">
      <small slot="help-text">Please enter "tibcowebcomponents" to continue</small>
    </TwcInput>

    <style>{css}</style>
  </>
);
```

### Third-party Validation

To opt out of the browser's built-in validation and use your own, add the `novalidate` attribute to the form. This will ignore all constraints and prevent the browser from showing its own warnings when form controls are invalid.

Remember that the `invalid` attribute on form controls reflects validity as defined by the [Constraint Validation API](https://developer.mozilla.org/en-US/docs/Web/Guide/HTML/HTML5/Constraint_validation). You can set it initially, but the `invalid` attribute will update as the user interacts with the form control. As such, you should not rely on it to set invalid styles using a custom validation library.

Instead, toggle a class and target it in your stylesheet as shown below.

```html
<form novalidate>
  <twc-input class="invalid"></twc-input>
</form>

<style>
  twc-input.invalid {
    ...;
  }
</style>
```
